<?php
defined('HOSTCMS') || exit('HostCMS: access denied.');

require_once CMS_FOLDER . 
            'modules' .  DIRECTORY_SEPARATOR . 
            'vtbpay' .  DIRECTORY_SEPARATOR . 
            'vendor'. DIRECTORY_SEPARATOR . 
            'autoload.php';

use \Vtbpay\Classes\VtbApi;

/**
 * Класс Shop_Payment_System_HandlerXX для обработки платежей через VTB
 *
 * Класс, предназначенный для интеграции с платежной системой VTB.
 * Этот обработчик позволяет настроить и обработать платежи через VTB в вашем интернет-магазине.
 *
 * @property string $client_id Идентификатор клиента (мерчанта)
 * @property string $client_secret Секретный ключ клиента (мерчанта)
 * @property string $merchant_authorization Идентификатор авторизации мерчанта
 * @property bool $test_mode Флаг для включения/выключения тестового режима
 * @property bool $enable_logging Флаг для включения/выключения логирования запросов к платежному шлюзу
 * @property bool $auto_redirect Флаг для включения/выключения автоматического редиректа после нажатия на кнопку оплатить
 * @property int $currency_id Идентификатор валюты
 * @property string $currency_code Код валюты
 * @property string $XSL_templates_invoice Шаблон XSL для квитанции
 * @property string $XSL_templates_notification Шаблон XSL для уведомления
 * 
 * @method __construct(Shop_Payment_System_Model $oShop_Payment_System_Model) Конструктор класса
 * @method userExecute() Возвращает строку с ссылкой перехода к оплате
 * @method getInvoice() Возвращает квитанцию
 * @method getNotification() Возвращает уведомление
 * @method getVtbApi() Возвращает экземпляр API VTB
 * @method getSumWithCoeff() Возвращает сумму товаров заказа с коэффициентом
 * @method checkPaymentBeforeContent() Метод, вызываемый в коде настроек ТДС через Shop_Payment_System_Handler::checkBeforeContent($oShop)
 * @method paymentProcessing() Обрабатывает уведомления об оплате заказа или возврате пользователя из магазина
 * @method _processOrder() Производит обработку нового заказа
 * @method log($level, $message) Логирование запроса и ответа от ПШ
 * 
 */
class Shop_Payment_System_HandlerXX extends Shop_Payment_System_Handler
{
  
    // Client ID мерчанта *
    private $client_id = ''; 

    // Client Secret мерчанта *
    private $client_secret = ''; 
    
    // Merchant-Authorization
    private $merchant_authorization = ''; 
    
    /**
     * Тестовый режим
     * TRUE - включить
     * FALSE - выключить
     */
    private $test_mode = TRUE;
    
    /**
     * Логирование запросов к платежному шлюзу
     * TRUE - включить
     * FALSE - выключить
     */
    private $enable_logging = TRUE;
    
    /**
     * Автоматический педирект после нажатия на кнопу оплатить
     * TRUE - включить
     * FALSE - выключить
     */
    protected $auto_redirect = TRUE;
    
    // Дефолтный идентификатор валюты
    protected $currency_id = 1;

    // Дефолтный код валюты
    protected $currency_code = 'RUB';
    
    // XSL-шаблон для накладной
    protected $XSL_templates_invoice = 'КвитанцияПД4';
    
    // XSL-шаблон для уведомления
    protected $XSL_templates_notification = 'ОплатаПоФормеПД4НовыйСайт2';
  
    
    /**
     * Конструктор класса.
     * При создании экземпляра класса, задает значения полям $currency_id и $currency_code, 
     * получая их из переданной модели платежной системы.
     * 
     * @param Shop_Payment_System_Model $oShop_Payment_System_Model Модель платежной системы
     */
    public function __construct(Shop_Payment_System_Model $oShop_Payment_System_Model)
    {
        parent::__construct($oShop_Payment_System_Model);

        $this->currency_id = $oShop_Payment_System_Model->shop_currency_id;

        $currency = Core_Entity::factory('Shop_Currency')->getById($this->currency_id);

        $this->currency_code = $currency->code;
    }
    
    
    /**
     * Возвращает html-строку с ссылкой для перехода к оплате.
     * Генерирует ссылку на оплату через VTB API и возвращает кнопку оплаты.
     * 
     * @return Shop_Payment_System_HandlerXX Возвращает экземпляр текущего объекта
     * @throws Exception Бросает исключение, если обязательные параметры пусты или не установлены
     */
    public function userExecute()
  	{
        if (empty($this->client_id)) {
            throw new Exception(Core::_('Vtbpay.client_id_is_empty'));
        }

        if (empty($this->client_secret)) {
            throw new Exception(Core::_('Vtbpay.client_secret_is_empty'));
        }
        
        $order_id = $this->getShopOrder()->id;
    
        // Get return url
        $oSite_Alias = $this->getShopOrder()->Shop->Site->getCurrentAlias();
        $site_alias = !is_null($oSite_Alias) ? $oSite_Alias->name : '';
        $shop_path = $this->getShopOrder()->Shop->Structure->getPath();
        $return_url = 'http://' . $site_alias . $shop_path . 'cart/?method=vtbpay&action=return';
        
        
        // Get Total
        $total = number_format($this->getSumWithCoeff(), 2, '.', '');
        
        
        // Get Email
        if(
            !isset($this->getShopOrder()->email) || 
            empty($email = $this->getShopOrder()->email)
        ) {
            $orderParams = $this->_orderParams;
            if(
                !isset($orderParams['email']) || 
                empty($email = $orderParams['email'])
            ) {
                throw new Exception(Core::_('Vtbpay.email_is_empty'));
            }
        }

        $vtbpay = $this->getVtbApi();
        $response = $vtbpay->getOrderLink(
            $order_id, 
            $email, 
            time(), 
            $total, 
            $return_url
        );
        
        $this->log('info', json_encode([
          'method_name' => 'userExecute',
          'request_method_name' => 'getOrderLink',
          'arguments' => [
            'order_id' => $order_id, 
            'email' => $email, 
            'time' => time(), 
            'total' => $total, 
            'return_url' => $return_url
          ],
          'response' => $response
        ]));
        
        $html = '';
        if (isset($response['object']['payUrl']) && !empty($pay_url = $response['object']['payUrl'])) {
            $html .= '<h2>' . Core::_('Vtbpay.go_to_payment_page') . ' #' . $order_id . '</h2>';
            $html .= '<a href="' . $pay_url . '" class="btn" id="button-confirm">' . Core::_('Vtbpay.pay_for_the_order') . '</a>';
            if ($this->auto_redirect) {
                $html .= '<script type="text/javascript">';
                $html .= 'const paymentButton = document.getElementById("button-confirm");';
                $html .= 'if (paymentButton) {';
                $html .= '    paymentButton.click();';
                $html .= '}';
                $html .= '</script>';
            }
        } 
        else {
            throw new \Exception(Core::_('Vtbpay.url_not_available'));
        }

        echo $html;
        
        return $this;
  	}
    
    
    /**
     * Получает и применяет XSL шаблон для счета.
     * 
     * @return Shop_Payment_System_HandlerXX Возвращает экземпляр текущего объекта
     */
    public function getInvoice()
    {
        $this->xsl(
          Core_Entity::factory('Xsl')->getByName($this->XSL_templates_invoice)
        );
        return parent::getInvoice();
    }
    
    
    /**
     * Получает и применяет XSL шаблон для уведомления.
     * 
     * @return Shop_Payment_System_HandlerXX Возвращает экземпляр текущего объекта
     */
    public function getNotification()
    {
        $this->xsl(
          Core_Entity::factory('Xsl')->getByName($this->XSL_templates_notification)
        );
        return parent::getNotification();
    }
    
    
    /**
     * Получает экземпляр API VTB.
     * Создает и возвращает новый экземпляр VtbApi, используя данные конфигурации.
     * 
     * @return VtbApi Возвращает экземпляр класса VtbApi
     */
    private function getVtbApi(): VtbApi 
    {
        return new VtbApi(
            $this->client_id,
            $this->client_secret,
            $this->test_mode,
            $this->merchant_authorization
        );
    }
    
    
    /**
     * Возвращает общую сумму товаров заказа с учетом коэффициента.
     * Если валюты заказа и платежной системы различаются, преобразует сумму в валюту платежной системы.
     * 
     * @return float Возвращает общую сумму заказа с учетом коэффициента
     */
    public function getSumWithCoeff()
    {
        if ($this->currency_id > 0 && $this->getShopOrder()->shop_currency_id > 0) {
            $sum = Shop_Controller::instance()->getCurrencyCoefficientInShopCurrency(
                $this->getShopOrder()->Shop_Currency,
                Core_Entity::factory('Shop_Currency', $this->currency_id)
            );
        } 
        else {
            $sum = 0;
        }

        return Shop_Controller::instance()->round($sum * $this->getShopOrder()->getAmount());
    }
    
    
    /**
     * Обрабатывает запрос перед отображением контента.
     * Если переданный метод равен 'vtbpay' и указан orderId, начинает обработку платежа.
     * 
     * @return void
     */
    public function checkPaymentBeforeContent()
    {
        $method = Core_Array::getGet('method');
        $order_id = intval(Core_Array::getGet('orderId'));
        
        if (
                $method == 'vtbpay' && 
                !empty($order_id)
        ) {
            $oShop_Order = Core_Entity::factory('Shop_Order')->find($order_id);
            
            if (!is_null($oShop_Order->id)) {
                Shop_Payment_System_Handler::factory($oShop_Order->Shop_Payment_System)
                    ->shopOrder($oShop_Order)
                    ->paymentProcessing();
            }
        }
    }
    
    
    /**
     * Обрабатывает уведомления об оплате заказа или возврат пользователя после оплаты.
     * Определяет, является ли запрос уведомлением об оплате или возвратом пользователя.
     * 
     * @return void
     */
    public function paymentProcessing()
    {
        $action = Core_Array::getGet('action');
        
        if ($action == 'return') {
            $order_id = $this->getShopOrder()->id;
            
            $vtbpay = $this->getVtbApi();
            $response = $vtbpay->getOrderInfo($order_id);
            
            $this->log('info', json_encode([
              'method_name' => 'paymentProcessing',
              'request_method_name' => 'getOrderInfo',
              'arguments' => [
                'order_id' => $order_id
              ],
              'response' => $response
            ]));
            
            if (
                isset($response['object']['status']['value']) && 
                $response['object']['status']['value'] == 'PAID'
            ) {
                $this->getShopOrder()->system_information = Core::_('Vtbpay.order_successfully_paid') . ' \n';
                $this->getShopOrder()->paid();
                // Установка XSL-шаблонов в соответствии с настройками в узле структуры
        				$this->setXSLs();
        				// Отправка писем клиенту и пользователю
        				$this->send();
                        
                ob_start();
        				$this->shopOrderBeforeAction(clone $this->getShopOrder())->changedOrder('changeStatusPaid');
        				ob_get_clean();
    
                $payment_status = 'success';
            }
            else {
                $this->getShopOrder()->system_information = Core::_('Vtbpay.payment_error') . ' \n';
                $this->getShopOrder()->save();
                
                $payment_status = 'fail';
            }
            
            $oSite_Alias = $this->getShopOrder()->Shop->Site->getCurrentAlias();
            $site_alias = !is_null($oSite_Alias) ? $oSite_Alias->name : '';
            $shop_path = $this->getShopOrder()->Shop->Structure->getPath();
            
            header("location: http://{$site_alias}{$shop_path}cart/?payment={$payment_status}&order_id={$order_id}");
            exit();
        }
    }
    
    
    /**
     * Обрабатывает новый заказ.
     * Вызывает стандартную обработку заказа из родительского класса, 
     * устанавливает XSL шаблоны и отправляет уведомления о новом заказе.
     * 
     * @return Shop_Payment_System_HandlerXX Возвращает экземпляр текущего объекта
     */
    protected function _processOrder()
    {
        // Вызываем стандартное оформление заказа из родительского класса Shop_Payment_System_Handler
        parent::_processOrder();

        // Установка XSL-шаблонов в соответствии с настройками в узле структуры
        $this->setXSLs();

        // Отправка писем клиенту и пользователю
        $this->send();

        return $this;
    }
    
    
    /**
     * Записывает логи запроса и ответа от платежной системы.
     * Если включено логирование, записывает сообщение в лог.
     * 
     * @param string $level Уровень логирования
     * @param string $message Сообщение для лога
     * @return void
     */
     private function log($level, $message)
     {
         if ($this->enable_logging) {
             VtbpayLogger::instance()->log($level, $message);
         }
     }
}


/**
 * Класс VtbpayLogger расширяет функциональность Core_Log, предоставляя логирование для Vtbpay.
 * 
 * @method static VtbpayLogger instance() Создает и возвращает экземпляр класса.
 * @method string getLogName($date) Возвращает название файла лога в зависимости от переданной даты.
 * @method void log($level, $message, $context = null) Записывает сообщение в лог с определенным уровнем.
 * @method int convertLevelToType($level) Конвертирует стандартный уровень логирования в тип HostCMS.
 */
 if(!class_exists('VtbpayLogger')) {
    class VtbpayLogger extends Core_Log
    {
      
       /**
        * Создает и возвращает экземпляр данного класса.
        * 
        * @return VtbpayLogger Возвращает новый экземпляр класса VtbpayLogger.
        */
        static public function instance()
        {
            return new self();
        }


        /**
         * Возвращает название файла лога, формируемое по переданной дате.
         * 
         * @param mixed $date Дата для формирования названия файла.
         * @return string Возвращает название файла лога.
         */
        public function getLogName($date)
        {
            return $this->_logDir . 
                    DIRECTORY_SEPARATOR . 
                    'vtbpay_log_' . 
                    date('d_m_Y', Core_Date::sql2timestamp($date)) . 
                    '.log';
        }
        

        /**
         * Записывает сообщение в лог с определенным уровнем.
         * Уровень логирования преобразуется в тип HostCMS и затем записывается в лог.
         * 
         * @param string $level Уровень логирования.
         * @param string $message Сообщение для записи.
         * @param mixed $context (Необязательно) Дополнительный контекст для сообщения.
         * @return void
         */
        public function log($level, $message, $context = null)
        {
            $this->clear()
                ->notify(false)
                ->status($this->convertLevelToType($level))
                ->write($message);
        }
        

        /**
         * Конвертирует стандартный уровень логирования в тип HostCMS.
         * 
         * @param string $level Уровень логирования для конвертации.
         * @return int Возвращает соответствующий тип HostCMS.
         */
        private function convertLevelToType ($level)
        {
            $type = self::$MESSAGE;

            switch ($level) {
                case 'info':
                    $type = self::$MESSAGE;
                    break;
                case 'debug':
                    $type = self::$SUCCESS;
                    break;
                case 'notice':
                    $type = self::$NOTICE;
                    break;
                case 'warn':
                    $type = self::$WARNING;
                    break;
                case 'error':
                    $type = self::$ERROR;
                    break;
            }

            return $type;
        }
    }
}